@@ -2,6 +2,27 @@
 Description
 ===========
 
+Parse command line arguments for Flask, or for Celery, and export
+the contents of the yaml configuration file passed by `--cfgfile` (Flask)
+or `--conf` (Celery) to the Flask app.
+
+No settings are stored directly in Celery app.
+
+When ArgParseExt is used on the Flask application, it will parse application
+arguments and display the help if needed.
+
+When used on the Celery application, it will parse application arguments
+only to find the `--conf` flag, equivalent to the Flask `--cfgfile` argument.
+
+ArgParseExt then searchs for the configuration file according to it's name,
+"settings.yml" by default. The default search path will be shown in the `--help`
+command printout.
+
+`default_cfg` and `forced_cfg` are relative to the root contents of the settings
+file. In other words, when passing Celery configuration, the celery settings must
+be under the `celery` key.
+
+
 .. docs/ext/argparse/flask.rst
 .. include:: argparse/flask.rst
 
@@ -14,15 +35,34 @@ Usage
 
 .. program:: flask-dev
 
+Initializing applications
+-------------------------
+
 .. code-block:: python
 
   #!/usr/bin/env python3
 
   from flask import Flask
   from flask_gordon.ext import ArgParseExt
+  from flask_gordon.ext import CeleryExt
 
-  flask = Flask(__name__)
-  flask = ArgParseExt(flask)
+  # Initialize Flask
+  app = Flask(__name__)
+
+  # Initialize Celery
+  celery = CeleryExt().init_app(app)
+  celery = ArgParseExt().init_app(
+      forced_cfg={
+          "imports": "<PACKAGE_NAME>.tasks",
+      },
+  )
+
+  # If this is Flask:
+  if __name__ == "__main__":
+      global app
+      app = ArgParseExt().init_app(app)
+      # This line is reached if parsing of arguments did
+      # not raise an error and was not `--help`.
 
 
 Classes
@@ -37,62 +77,38 @@ import os
 import sys
 import typing as t
 
-from torxtools import xdgtools
 from torxtools.argtools import is_not_dir
-from torxtools.cfgtools import which
-from torxtools.pathtools import cachedir, expandpath
-from yaml import safe_load
 
-from ._utils import deepmerge
-from .defaults import CFGFILE_SEARCH_PATHS, make_config
+from .. import functions
+from ._functions import deepmerge, prepare_configuration, read_configuration
 from .flask import FlaskHelper
 from .gunicorn import GunicornHelper
 
 
-def _read(cfgfile: str) -> t.Dict[str, t.Any]:
-    """
-    Convenience function in order to be mocked.
-
-    Parameters
-    ----------
-    cfgfile: str
-
-        a single path representing a yaml file.
-
-    Returns
-    -------
-    dict:
-
-        a dictionary
-    """
-    with open(cfgfile, encoding="UTF-8") as fd:
-        data = safe_load(fd)
-    return data or {}
-
-
 class ArgParseExt:
     def __init__(
         self,
-        app: "FlaskApp" = None,
+        app=None,
         middleware: str = "gunicorn",
         argv: t.List[str] = None,
         cfgfilename=None,
         cfgfilepaths=None,
         default_cfg: t.Dict[str, t.Any] = None,
+        forced_cfg: t.Dict[str, t.Any] = None,
         description="Flasket server",
-        key: str = "SETTINGS",
         appname: str = None,
     ):
         """
         Parameters
         ----------
-        app: FlaskApp
+        app:
 
-            A Flask application.
+            A Flask or Celery application.
 
         middleware: str
 
             Middleware to use: 'gunicorn' for production, 'flask' for debug.
+            Unused in the case of a Celery app.
 
         argv: list[str]
 
@@ -112,32 +128,122 @@ class ArgParseExt:
             Passed value will be merged with the default factory configuration, command line arguments
             and the configuration file that was read.
 
+        forced_cfg: dict, default: {}
+
+            Dictionary containing the forced settings.
+            Passed value will be merged with the default factory configuration, command line arguments
+            and the configuration file that was read, and then the forced_cfg will overwrite the values.
+            Flask middleware forced configuration will overwrite the final configuration.
+
         description: str
 
             Text description to use in argparse.
-
-        key: str
-
-            Keyname where to save settings, defaults to app.config["SETTINGS"]
+            Unused in the case of a Celery app.
 
         appname: str
 
-            Application name. Used to figure out cachepath. Otherwise, defaults to sys.argv[0]
+            Application name. Used to show application name in `--help` string.
+            Defaults to sys.argv[0]
+            Unused in the case of a Celery app.
+
         """
         if app is not None:
             self.init_app(
-                app=app,
+                app,
+                middleware,
                 argv=argv,
-                middleware=middleware,
                 cfgfilename=cfgfilename,
                 cfgfilepaths=cfgfilepaths,
                 default_cfg=default_cfg,
+                forced_cfg=forced_cfg,
                 description=description,
-                key=key,
                 appname=appname,
             )
 
     def init_app(
+        self,
+        app,
+        middleware: str = "gunicorn",
+        argv: t.List[str] = None,
+        cfgfilename=None,
+        cfgfilepaths=None,
+        default_cfg: t.Dict[str, t.Any] = None,
+        forced_cfg: t.Dict[str, t.Any] = None,
+        description="Flask server",
+        appname: str = None,
+    ):
+        # Is this a flask or celery application?
+        app_class = functions.get_app_class(app)
+        if app_class == "flask":
+            return self.init_flask_app(
+                app,
+                middleware,
+                argv=argv,
+                cfgfilename=cfgfilename,
+                cfgfilepaths=cfgfilepaths,
+                default_cfg=default_cfg,
+                forced_cfg=forced_cfg,
+                description=description,
+                appname=appname,
+            )
+
+        if app_class == "celery":
+            return self.init_celery_app(
+                app,
+                argv=argv,
+                cfgfilename=cfgfilename,
+                cfgfilepaths=cfgfilepaths,
+                default_cfg=default_cfg,
+                forced_cfg=forced_cfg,
+            )
+
+        raise NotImplementedError("Could not determine if app is Flask or Celery")
+
+    def init_celery_app(
+        self,
+        app: "Celery",
+        argv: t.List[str] = None,
+        cfgfilename=None,
+        cfgfilepaths=None,
+        default_cfg: t.Dict[str, t.Any] = None,
+        forced_cfg: t.Dict[str, t.Any] = None,
+    ):
+        celery_app = app
+
+        cfgfilename, cfgfilepaths, defaults = prepare_configuration(cfgfilename, cfgfilepaths, default_cfg)
+
+        # Parse arguments if they exist
+        if argv is None:
+            argv = sys.argv[1:]
+
+        cfgfile = None
+        if "--config" in argv:
+            idx = argv.index("--config")
+            cfgfile = argv[idx + 1]
+
+        data = read_configuration(cfgfile, cfgfilename, cfgfilepaths)
+
+        # Merge in the reverse order of priority
+        cfg = defaults
+        cfg = deepmerge(cfg, data)
+        cfg = deepmerge(cfg, forced_cfg or {})
+
+        if "celery" not in cfg:
+            cfg["celery"] = {}
+
+        # Copy the configuration to the app.
+        # When celery starts, the ArgParseExt of app is not called since argv differs
+        # between starting the Flask application and the Celery application
+        #
+        if getattr(celery_app, "app"):
+            celery_app.app().config["SETTINGS"] = cfg
+
+        # Create a celery application
+        celery_app.config_from_object(cfg["celery"])
+        celery_app.set_default()
+        return celery_app
+
+    def init_flask_app(
         self,
         app: "Flask",
         middleware: str = "gunicorn",
@@ -145,12 +251,11 @@ class ArgParseExt:
         cfgfilename=None,
         cfgfilepaths=None,
         default_cfg: t.Dict[str, t.Any] = None,
+        forced_cfg: t.Dict[str, t.Any] = None,
         description="Flask server",
-        key: str = "SETTINGS",
         appname: str = None,
     ):
-        # Sets environment variables for XDG paths
-        xdgtools.setenv()
+        flask_app = app
 
         # Verify middleware exists
         middleware = middleware.lower().strip()
@@ -161,24 +266,14 @@ class ArgParseExt:
         if middleware == "gunicorn":
             helper = GunicornHelper()
 
-        # Prepare search path for configuration file. Disable it if we're looking
-        # for a impossible (None) file name
-        if not cfgfilename:
-            cfgfilename = "flask.yml"
-        if cfgfilepaths is None:
-            cfgfilepaths = CFGFILE_SEARCH_PATHS
-        cfgfilepaths = [e.format(cfgfilename=cfgfilename) for e in cfgfilepaths]
-
-        # Create a default configuration from what was passed
-        # and what we set. Other values are filtered
-        defaults = make_config(default_cfg)
-
         # Parse arguments if they exist
         if argv is None:
             argv = sys.argv[1:]
             if appname is None:
                 appname = os.path.basename(sys.argv[0])
 
+        cfgfilename, cfgfilepaths, defaults = prepare_configuration(cfgfilename, cfgfilepaths, default_cfg)
+
         arguments = self._parse_arguments(
             argv=argv,
             cfgfilepaths=cfgfilepaths,
@@ -192,27 +287,17 @@ class ArgParseExt:
         cfgfile = arguments.pop("cfgfile", None)
         arguments = {"server": arguments}
 
-        # Search for the configuration file
-        if cfgfile is None and cfgfilepaths:
-            # Search for cfgfile
-            cfgfilepaths = [e.format(cfgfilename=cfgfilename) for e in cfgfilepaths]
-            cfgfile = which(cfgfile, expandpath(cfgfilepaths))
-
-        filedata = _read(cfgfile or "/dev/null") or {}
+        data = read_configuration(cfgfile, cfgfilename, cfgfilepaths)
 
         # Merge in the reverse order of priority
         cfg = defaults
-        cfg = deepmerge(cfg, filedata)
+        cfg = deepmerge(cfg, data)
         cfg = deepmerge(cfg, arguments)
+        cfg = deepmerge(cfg, forced_cfg or {})
         cfg = helper.force_cfg(cfg)
 
-        app.config[key] = cfg
-
-        # Set cachdir from configuration or default
-        path = cfg.get("server", {}).get("cachedir", {})
-        app.cachedir = cachedir(appname, path)
-
-        return app
+        flask_app.config["SETTINGS"] = cfg
+        return flask_app
 
     @staticmethod
     def _parse_arguments(*, argv, cfgfilepaths, helper, defaults, description):
