@@ -53,7 +53,7 @@ class FlaskHelper:
     @staticmethod
     def force_cfg(cfg: t.Dict["str", t.Any]) -> None:
         # debug is set via flag
-        cfg["production"] = False
+        cfg["server"]["production"] = False
         return cfg
 
     @staticmethod
